@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: client.c,v 1.6 2005/04/19 03:40:33 evertonm Exp $ */
+/* $Id: client.c,v 1.17 2005/07/27 12:01:46 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -33,7 +33,8 @@ Boston, MA 02111-1307, USA.
 #include "pipe.h"
 #include "common.h"
 
-nepim_pipe_set_t pipes;
+extern nepim_pipe_set_t pipes; /* from server.c */
+extern nepim_slot_set_t slots; /* from server.c */
 
 static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user);
 static void tcp_pipe_stop(int sd);
@@ -42,17 +43,19 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user);
 static void schedule_stat_interval(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_session_t *session;
   assert(pipe);
+  session = &pipe->session;
 
   {
-    int result = gettimeofday(&pipe->tv_interval, 0);
+    int result = gettimeofday(&session->tv_interval, 0);
     assert(!result);
   }
 
-  pipe->tv_interval.tv_sec += pipe->stat_interval;
+  session->tv_interval.tv_sec += session->stat_interval;
 
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				pipe->tv_interval,
+				session->tv_interval,
 				on_tcp_interval, pipe);
 }
 
@@ -62,6 +65,7 @@ static void *on_tcp_read(oop_source *src, int sd,
   char buf[nepim_global.tcp_read_size];
   int rd;
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session;
 
   assert(event == OOP_READ);
   assert(sd == pipe->sd);
@@ -87,10 +91,12 @@ static void *on_tcp_read(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  pipe->byte_total_recv += rd;
-  pipe->byte_interval_recv += rd;
-  ++pipe->total_reads;
-  ++pipe->interval_reads;
+  session = &pipe->session;
+
+  session->byte_total_recv += rd;
+  session->byte_interval_recv += rd;
+  ++session->total_reads;
+  ++session->interval_reads;
 
   return OOP_CONTINUE;
 }
@@ -101,10 +107,11 @@ static void *on_tcp_write(oop_source *src, int sd,
   char buf[nepim_global.tcp_write_size];
   int wr;
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
-  assert(pipe->max_bit_rate < 1);
+  assert(session->max_bit_rate < 1);
 
   wr = write(sd, buf, nepim_global.tcp_write_size);
   if (wr < 1) {
@@ -130,10 +137,10 @@ static void *on_tcp_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  pipe->byte_total_sent += wr;
-  pipe->byte_interval_sent += wr;
-  ++pipe->total_writes;
-  ++pipe->interval_writes;
+  session->byte_total_sent += wr;
+  session->byte_interval_sent += wr;
+  ++session->total_writes;
+  ++session->interval_writes;
 
   return OOP_CONTINUE;
 }
@@ -144,13 +151,14 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   char buf[nepim_global.tcp_write_size];
   int wr;
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
   int to_write;
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
-  assert(pipe->max_bit_rate > 0);
+  assert(session->max_bit_rate > 0);
 
-  to_write = NEPIM_MIN(pipe->rate_remaining, nepim_global.tcp_write_size);
+  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.tcp_write_size);
 
   wr = write(sd, buf, to_write);
   if (wr < 1) {
@@ -176,28 +184,28 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  pipe->byte_total_sent += wr;
-  pipe->byte_interval_sent += wr;
-  pipe->rate_remaining -= wr;
-  ++pipe->total_writes;
-  ++pipe->interval_writes;
+  session->byte_total_sent += wr;
+  session->byte_interval_sent += wr;
+  session->rate_remaining -= wr;
+  ++session->total_writes;
+  ++session->interval_writes;
 
 #if 0
   fprintf(stderr, 
 	  "DEBUG %s %s %d: to_write=%d written=%d missing=%d\n",
 	  __FILE__, __PRETTY_FUNCTION__,
-	  sd, to_write, wr, pipe->rate_remaining);
+	  sd, to_write, wr, session->rate_remaining);
 #endif
 
   /* finished ? */
-  if (pipe->rate_remaining < 1) {
+  if (session->rate_remaining < 1) {
     /* stop writing */
     nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				    sd, OOP_WRITE);
 
     /* schedule for next time saved by on_tcp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  pipe->tv_rate,
+				  session->tv_rate,
 				  on_tcp_rate_delay, pipe);
   }
 
@@ -207,27 +215,28 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
   long long tmp;
 
-  assert(timercmp(&tv, &pipe->tv_rate, ==));
-  assert(pipe->max_bit_rate > 0);
+  assert(timercmp(&tv, &session->tv_rate, ==));
+  assert(session->max_bit_rate > 0);
 
   /* save next scheduling time */
   {
-    int result = gettimeofday(&pipe->tv_rate, 0);
+    int result = gettimeofday(&session->tv_rate, 0);
     assert(!result);
   }
-  pipe->tv_rate.tv_usec += pipe->write_delay;
-  if (pipe->tv_rate.tv_usec >= 1000000) {
-    pipe->tv_rate.tv_usec -= 1000000;
-    ++pipe->tv_rate.tv_sec;
+  session->tv_rate.tv_usec += session->write_delay;
+  if (session->tv_rate.tv_usec >= 1000000) {
+    session->tv_rate.tv_usec -= 1000000;
+    ++session->tv_rate.tv_sec;
   }
 
   /* calculate bytes to be written from rate */
-  tmp = pipe->max_bit_rate;
-  tmp *= pipe->write_delay;
+  tmp = session->max_bit_rate;
+  tmp *= session->write_delay;
   tmp /= 8000000;
-  pipe->rate_remaining = tmp;
+  session->rate_remaining = tmp;
 
 #if 0
   fprintf(stderr, 
@@ -235,10 +244,10 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 	  "(bit_rate * delay / 8M = bytes) "
 	  "%d * %ld / 8M = %d\n",
 	  __FILE__, __PRETTY_FUNCTION__,
-	  pipe->sd, 
-	  pipe->max_bit_rate,
-	  pipe->write_delay, 
-	  pipe->rate_remaining);
+	  session->sd, 
+	  session->max_bit_rate,
+	  session->write_delay, 
+	  session->rate_remaining);
 #endif
 
   /* start to write */
@@ -252,22 +261,23 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
 
-  assert(timercmp(&tv, &pipe->tv_duration, ==));
+  assert(timercmp(&tv, &session->tv_duration, ==));
 
-  nepim_dump_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
-		  pipe->byte_total_recv,
-		  pipe->byte_total_sent,
-		  pipe->test_duration,
-		  pipe->tv_start.tv_sec,
-		  pipe->test_duration,
-		  pipe->total_reads,
-		  pipe->total_writes);
+  nepim_pipe_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
+		  session->byte_total_recv,
+		  session->byte_total_sent,
+		  session->test_duration,
+		  session->tv_start.tv_sec,
+		  session->test_duration,
+		  session->total_reads,
+		  session->total_writes);
 
-  pipe->byte_total_recv = 0;
-  pipe->byte_total_sent = 0;
-  pipe->total_reads     = 0;
-  pipe->total_writes    = 0;
+  session->byte_total_recv = 0;
+  session->byte_total_sent = 0;
+  session->total_reads     = 0;
+  session->total_writes    = 0;
 
   tcp_pipe_stop(pipe->sd);
   close(pipe->sd);
@@ -278,22 +288,23 @@ static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
 static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
 
-  assert(timercmp(&tv, &pipe->tv_interval, ==));
+  assert(timercmp(&tv, &session->tv_interval, ==));
 
-  nepim_dump_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
-		  pipe->byte_interval_recv,
-		  pipe->byte_interval_sent,
-		  pipe->stat_interval,
-		  pipe->tv_start.tv_sec,
-		  pipe->test_duration,
-		  pipe->interval_reads,
-		  pipe->interval_writes);
+  nepim_pipe_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
+		  session->byte_interval_recv,
+		  session->byte_interval_sent,
+		  session->stat_interval,
+		  session->tv_start.tv_sec,
+		  session->test_duration,
+		  session->interval_reads,
+		  session->interval_writes);
 
-  pipe->byte_interval_recv = 0;
-  pipe->byte_interval_sent = 0;
-  pipe->interval_reads     = 0;
-  pipe->interval_writes    = 0;
+  session->byte_interval_recv = 0;
+  session->byte_interval_sent = 0;
+  session->interval_reads     = 0;
+  session->interval_writes    = 0;
 
   schedule_stat_interval(pipe->sd);
 
@@ -310,7 +321,7 @@ static int greet(int sd)
 		"hello server_send=%d bit_rate=%lld "
 		"stat_interval=%d test_duration=%d "
 		"write_delay=%ld\n", 
-		nepim_global.duplex_mode,
+		nepim_global.duplex_mode || !nepim_global.simplex_client_send,
 		nepim_global.bit_rate,
 		nepim_global.stat_interval,
 		nepim_global.test_duration,
@@ -330,17 +341,20 @@ static int greet(int sd)
 static void tcp_pipe_start(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_session_t *session;
   assert(pipe);
 
+  session = &pipe->session;
+
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
 			      sd, OOP_READ,
 			      on_tcp_read, pipe);
 
-  if (pipe->must_send) {
-    if (pipe->max_bit_rate > 0) {
-      pipe->tv_rate = OOP_TIME_NOW;
+  if (session->must_send) {
+    if (session->max_bit_rate > 0) {
+      session->tv_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    pipe->tv_rate,
+				    session->tv_rate,
 				    on_tcp_rate_delay, pipe);
     }
     else {
@@ -350,37 +364,41 @@ static void tcp_pipe_start(int sd)
     }
   }
 
-
   {
-    int result = gettimeofday(&pipe->tv_start, 0);
+    int result = gettimeofday(&session->tv_start, 0);
     assert(!result);
-    pipe->tv_duration = pipe->tv_start;
+    session->tv_duration = session->tv_start;
   }
-  pipe->tv_duration.tv_sec += pipe->test_duration;
+  session->tv_duration.tv_sec += session->test_duration;
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				pipe->tv_duration,
+				session->tv_duration,
 				on_tcp_duration, pipe);
 
+  nepim_sock_show_opt(stderr, sd);
+
   schedule_stat_interval(sd);
 }
 
 static void tcp_pipe_stop(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_session_t *session;
   assert(pipe);
 
+  session = &pipe->session;
+
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				  sd, OOP_READ);
 
-  if (pipe->must_send) {
-    if (pipe->max_bit_rate > 0) {
+  if (session->must_send) {
+    if (session->max_bit_rate > 0) {
       /* stop current writing, if any */
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				      sd, OOP_WRITE);
 
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					pipe->tv_rate,
+					session->tv_rate,
 					on_tcp_rate_delay, pipe);
     }
     else {
@@ -390,14 +408,16 @@ static void tcp_pipe_stop(int sd)
   }
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    pipe->tv_duration,
+				    session->tv_duration,
 				    on_tcp_duration, pipe);
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    pipe->tv_interval,
+				    session->tv_interval,
 				    on_tcp_interval, pipe);
 
   nepim_pipe_set_del(&pipes, sd);
+
+  nepim_sock_show_opt(stderr, sd);
 }
 
 static void spawn_one_tcp_client()
@@ -416,13 +436,15 @@ static void spawn_one_tcp_client()
   hints.ai_canonname = 0;
 
   fprintf(stderr, 
-	  "TCP socket solving %s,%d\n",
-	  nepim_global.hostname, nepim_global.port);
+	  "TCP socket solving %s,%s\n",
+	  nepim_global.hostname, nepim_global.portname);
 
-  result = getaddrinfo(nepim_global.hostname, 0, &hints, &ai_res);
+  result = getaddrinfo(nepim_global.hostname, nepim_global.portname,
+		       &hints, &ai_res);
   if (result) {
-    fprintf(stderr, "getaddrinfo(%s): %s\n",
-            nepim_global.hostname, gai_strerror(result));
+    fprintf(stderr, "getaddrinfo(%s, %s): %s\n",
+            nepim_global.hostname, nepim_global.portname,
+	    gai_strerror(result));
     return;
   }
 
@@ -430,31 +452,42 @@ static void spawn_one_tcp_client()
     int sd;
 
     fprintf(stderr, 
-	    "TCP socket trying %s,%d\n",
-	    nepim_global.hostname, nepim_global.port);
+	    "TCP socket trying %s,%s\n",
+	    nepim_global.hostname, nepim_global.portname);
+
+    if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
+      continue;
 
     sd = nepim_connect_client_socket(ai->ai_addr, ai->ai_addrlen,
 				     ai->ai_family, ai->ai_socktype, 
-				     ai->ai_protocol, nepim_global.port);
+				     ai->ai_protocol,
+				     nepim_global.pmtu_mode,
+				     nepim_global.ttl);
     if (sd < 0) {
       fprintf(stderr, 
-	      "could not connect TCP socket to %s,%d: %d\n",
-	      nepim_global.hostname, nepim_global.port, sd);
+	      "could not connect TCP socket to %s,%s: %d\n",
+	      nepim_global.hostname, nepim_global.portname, sd);
       continue;
     }
 
     fprintf(stderr, 
 	    "%d: TCP socket connected to: "
-	    "host=%s,%d len=%d family=%d type=%d proto=%d\n",
-	    sd, nepim_global.hostname, nepim_global.port, 
+	    "host=%s,%s len=%d family=%d type=%d proto=%d\n",
+	    sd, nepim_global.hostname, nepim_global.portname, 
 	    ai->ai_addrlen, ai->ai_family, 
 	    ai->ai_socktype, ai->ai_protocol);
 
-    nepim_pipe_set_add(&pipes, sd, 1,
-		       nepim_global.bit_rate, 
-		       nepim_global.stat_interval,
-		       nepim_global.test_duration,
-		       nepim_global.write_delay);
+    {
+      nepim_greet_t opt;
+
+      opt.must_send     = nepim_global.duplex_mode || nepim_global.simplex_client_send;
+      opt.bit_rate      = nepim_global.bit_rate;
+      opt.stat_interval = nepim_global.stat_interval;
+      opt.test_duration = nepim_global.test_duration;
+      opt.write_delay   = nepim_global.write_delay;
+
+      nepim_pipe_set_add(&pipes, sd, &opt);
+    }
 
     break;
   }
@@ -471,13 +504,13 @@ static void spawn_tcp_clients()
     spawn_one_tcp_client();
 
   /* activate sockets */
-  for (i = 0; i < pipes.capacity; ++i)
+  for (i = 0; i < pipes.array.capacity; ++i)
     if (nepim_pipe_set_get(&pipes, i)) {
 
       if (greet(i)) {
 	fprintf(stderr,
-		"%d: could not greet %s,%d\n",
-		i, nepim_global.hostname, nepim_global.port);
+		"%d: could not greet %s,%s\n",
+		i, nepim_global.hostname, nepim_global.portname);
 
 	nepim_pipe_set_del(&pipes, i);
 	close(i);
@@ -486,8 +519,8 @@ static void spawn_tcp_clients()
       }
 
       fprintf(stderr, 
-	      "%d: greetings sent to %s,%d\n",
-	      i, nepim_global.hostname, nepim_global.port);
+	      "%d: greetings sent to %s,%s\n",
+	      i, nepim_global.hostname, nepim_global.portname);
 
       tcp_pipe_start(i);
     }
@@ -499,10 +532,12 @@ void nepim_client_run()
 {
   void *result;
 
-  nepim_pipe_set_init(&pipes);
+  nepim_pipe_set_init(&pipes); /* TCP/UDP sockets storage */
 
-  if (nepim_global.udp_mode)
+  if (nepim_global.udp_mode) {
+    nepim_slot_set_init(&slots); /* UDP pseudo-streams */
     nepim_udp_clients();
+  }
   else
     spawn_tcp_clients();
 
