@@ -19,28 +19,34 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: common.h,v 1.2 2005/04/19 03:40:33 evertonm Exp $ */
+/* $Id: common.h,v 1.8 2005/07/27 12:01:46 evertonm Exp $ */
 
 #ifndef NEPIM_COMMON_H
 #define NEPIM_COMMON_H
 
 #include "pipe.h"
+#include "slot.h"
 
 #define NEPIM_MIN(a, b) (((a) < (b)) ? (a) : (b))
 
-
 extern const char * const NEPIM_LABEL_PARTIAL;
 extern const char * const NEPIM_LABEL_TOTAL;
 
-
 const char *nepim_version();
 
-void nepim_dump_stat(FILE *out, const char *label, int sd, 
+void nepim_pipe_stat(FILE *out, const char *label, int sd,
 		     long long bytes_recv, long long bytes_sent, 
-		     float interval, long sec_start,
+		     float interval, long sec_start, 
+		     long sec_duration, int reads, int writes);
+
+void nepim_slot_stat(FILE *out, const char *label, int sd,
+		     int local_slot, int remote_slot,
+		     long long bytes_recv, long long bytes_sent, 
+		     float interval, long sec_start, 
 		     long sec_duration, int reads, int writes);
 
 void report_broken_pipe_stat(FILE *out, const nepim_pipe_t *pipe);
+void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot);
 
 
 #endif /* NEPIM_COMMON_H */
