@@ -22,22 +22,29 @@ Boston, MA 02111-1307, USA.
 #include "conf.h"
 
 nepim_global_t nepim_global = {
-  0,      /* prog_name */
-  0,      /* hostname */
-  0,      /* client_mode */
-  0,      /* udp_mode */
-  0,      /* duplex_mode */
-  1,      /* pipes */
-  -1,     /* bit_rate */
-  2,      /* stat_interval */
-  10,     /* test_duration */
-  32768,  /* tcp_write_size */
-  32768,  /* tcp_read_size */
-  4096,   /* udp_write_size */
-  4096,   /* udp_read_size */
-  1234,   /* port */
-  0,      /* no_inet6 */
-  250000, /* write_delay (usec, bit_rate > 0) */
+  "nepim", /* prog_name */
+  0,       /* hostname */
+  0,       /* client_mode */
+  0,       /* udp_mode */
+  0,       /* simplex_client_send */
+  0,       /* duplex_mode */
+  1,       /* pipes */
+  -1,      /* bit_rate */
+  2,       /* stat_interval */
+  10,      /* test_duration */
+  32768,   /* tcp_write_size */
+  32768,   /* tcp_read_size */
+  4096,    /* udp_write_size */
+  4096,    /* udp_read_size */
+  "1234",  /* portname */
+  0,       /* no_inet6 */
+  250000,  /* write_delay (usec, bit_rate > 0) */
+  100,     /* listen_backlog */
+  -1,      /* pmtu_mode */
+  0,       /* bind_list */
+  0,       /* join_list */
+  -1,      /* ttl */
+  -1,      /* mcast_ttl */
 
   0, /* oop source */
   0  /* source interface */
