@@ -31,6 +31,7 @@ struct nepim_global_t {
   const char *hostname;
   int         client_mode;
   int         udp_mode;
+  int         simplex_client_send;
   int         duplex_mode;
   int         pipes;
   long long   bit_rate;
@@ -40,9 +41,15 @@ struct nepim_global_t {
   int         tcp_read_size;
   int         udp_write_size;
   int         udp_read_size;
-  int         port;
+  const char *portname;
   int         no_inet6;
   suseconds_t write_delay;
+  int         listen_backlog;
+  int         pmtu_mode;
+  char       *bind_list;
+  char       *join_list;
+  int         ttl;
+  int         mcast_ttl;
 
   oop_source_sys *oop_sys;
   oop_source     *oop_src;
