@@ -21,7 +21,7 @@ Boston, MA 02111-1307, USA.
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.7 2005/07/08 20:04:55 evertonm Exp $
+  $Id: main.c,v 1.13 2005/07/27 13:31:45 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -41,9 +41,10 @@ typedef void (*sighandler_t)(int);
 
 #include "conf.h"
 #include "common.h"
+#include "str.h"
 
-void nepim_server_run();
-void nepim_client_run();
+extern void nepim_server_run();
+extern void nepim_client_run();
 
 
 static void usage(FILE *out, const char *prog_name)
@@ -64,27 +65,40 @@ static void usage(FILE *out, const char *prog_name)
 	  "usage: %s [options]\n"
 	  "\n"
 	  "common (client/server) options:\n"
-	  "-h         help\n"
-	  "-v         show program version\n"
-	  "-p port    server port number (defaults to %d)\n"
-	  "-b address bind to specific local address (defaults to any)\n"
-	  "-w size    TCP write size (defaults to %d bytes)\n"
-	  "-z size    TCP read size (defaults to %d bytes)\n"
-	  "-W size    UDP write size (defaults to %d bytes)\n"
-	  "-Z size    UDP read size (defaults to %d bytes)\n"
-	  "-6         disable IPv6\n"
-	  "\n"
+	  "  -h      help\n"
+	  "  -v      show program version\n"
+	  "  -p port server port number (defaults to %s)\n"
+	  "  -m mode path MTU discovery mode (defaults to %d)\n"
+	  "          modes: -1 = use system-wide settings\n"
+	  "                  0 = DONT (never do PMTU discovery, DF=0)\n"
+	  "                  1 = WANT (use per-route hints)\n"
+	  "                  2 = DO   (always do PMTU discovery, DF=1)\n"
+	  "  -t ttl  TTL for sending packets (defaults to system)\n"
+	  "  -w size TCP write size (defaults to %d bytes)\n"
+	  "  -z size TCP read size (defaults to %d bytes)\n"
+	  "  -W size UDP write size (defaults to %d bytes)\n"
+	  "  -Z size UDP read size (defaults to %d bytes)\n"
+	  "  -6      disable IPv6\n"
 	  "client-only options:\n"
-	  "-u             udp mode (defaults to tcp)\n"
-	  "-d             duplex traffic (defaults to simplex)\n"
-	  "-c server-host client mode (defaults to server mode)\n"
-	  "-n pipes       number of parallel pipes (defaults to %d)\n"
-	  "-r rate        upper bit rate limit (defaults to %s bps)\n"
-	  "-i interval    statistics report interval (defaults to %d seconds)\n"
-	  "-a age         test duration (defaults to %d seconds)\n"
+	  "  -u             udp mode (defaults to tcp)\n"
+	  "  -s             client-send simplex mode (defaults to server-send)\n"
+	  "  -d             duplex traffic (defaults to simplex)\n"
+	  "  -c server-host client mode (defaults to server mode)\n"
+	  "  -n pipes       number of parallel pipes (defaults to %d)\n"
+	  "  -r rate        upper bit rate limit (defaults to %s bps)\n"
+	  "  -i interval    statistics report interval (defaults to %d seconds)\n"
+	  "  -a age         test duration (defaults to %d seconds)\n"
+	  "server-only options:\n"
+	  "  -b addr-list bind to specific addresses (defaults to any)\n"
+	  "  -j addr-list join multicast groups (defaults to empty)\n"
+	  "               addr-list: addr_1[,port_1][/.../addr_n[,port_n]]\n"
+	  "               example:   10.0.0.1/192.168.0.1,5000/::1\n"
+	  "  -l backlog   TCP listen(2) backlog (defaults to %d)\n"
+	  "  -T mcast-ttl TTL for multicast packets (defaults to system)\n"
 	  ,
 	  nepim_global.prog_name, 
-	  nepim_global.port,
+	  nepim_global.portname,
+	  nepim_global.pmtu_mode,
 	  nepim_global.tcp_write_size,
 	  nepim_global.tcp_read_size,
 	  nepim_global.udp_write_size,
@@ -92,7 +106,8 @@ static void usage(FILE *out, const char *prog_name)
 	  nepim_global.pipes,
 	  bit_rate,
 	  nepim_global.stat_interval,
-	  nepim_global.test_duration
+	  nepim_global.test_duration,
+	  nepim_global.listen_backlog
 	  );
 }
 
@@ -132,9 +147,9 @@ static void show_version()
 	 );
 }
 
+
 int main(int argc, const char *argv[])
 {
-  nepim_global.prog_name = argv[0];
   int i;
 
   for (i = 1; i < argc; ++i) {
@@ -156,6 +171,17 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-b")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing bind address list\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.bind_list = addr_list_append(nepim_global.bind_list, argv[i]);
+      continue;
+    }
+
     if (!strcmp(arg, "-c")) {
       ++i;
       if (i >= argc) {
@@ -189,6 +215,17 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-j")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing multicast join list\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.join_list = addr_list_append(nepim_global.join_list, argv[i]);
+      continue;
+    }
+
     if (!strcmp(arg, "-n")) {
       ++i;
       if (i >= argc) {
@@ -207,7 +244,18 @@ int main(int argc, const char *argv[])
 		nepim_global.prog_name);
 	exit(1);
       }
-      nepim_global.port = atoi(argv[i]);
+      nepim_global.portname = argv[i];
+      continue;
+    }
+
+    if (!strcmp(arg, "-m")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing PMTU discovery mode\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.pmtu_mode = atoi(argv[i]);
       continue;
     }
 
@@ -222,6 +270,33 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-s")) {
+      nepim_global.simplex_client_send = 1;
+      continue;
+    }
+
+    if (!strcmp(arg, "-t")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing ttl\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.ttl = atoi(argv[i]);
+      continue;
+    }
+
+    if (!strcmp(arg, "-T")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing multicast-ttl\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.mcast_ttl = atoi(argv[i]);
+      continue;
+    }
+
     if (!strcmp(arg, "-u")) {
       nepim_global.udp_mode = 1;
       continue;
@@ -276,6 +351,17 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-l")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing listen backlog\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.listen_backlog = atoi(argv[i]);
+      continue;
+    }
+
     fprintf(stderr, "%s: unknown option: %s\n",
 	    nepim_global.prog_name, arg);
     usage(stderr, nepim_global.prog_name);
@@ -300,6 +386,8 @@ int main(int argc, const char *argv[])
 	      nepim_global.tcp_write_size);
 
     nepim_client_run();
+
+    fprintf(stderr, "%s: done\n", nepim_global.prog_name);
     exit(0);
   }
 
@@ -311,5 +399,7 @@ int main(int argc, const char *argv[])
 	  nepim_global.udp_write_size);
 
   nepim_server_run();
+
+  fprintf(stderr, "%s: done\n", nepim_global.prog_name);
   exit(0);
 }
