@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: server.c,v 1.10 2005/05/30 22:07:14 evertonm Exp $ */
+/* $Id: server.c,v 1.22 2005/07/27 12:01:46 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,8 +34,13 @@ Boston, MA 02111-1307, USA.
 #include "sock.h"
 #include "pipe.h"
 #include "common.h"
+#include "slot.h"
+#include "usock.h"
+#include "str.h"
 
-nepim_pipe_set_t pipes;
+nepim_pipe_set_t  pipes;   /* tcp socket table */
+nepim_slot_set_t  slots;   /* udp pseudo-sockets */
+nepim_usock_set_t udp_tab; /* udp socket table */
 
 const char *INET_ANY = "0.0.0.0";
 const char *INET6_ANY = "::";
@@ -52,6 +57,7 @@ static void *on_tcp_read(oop_source *src, int sd,
   char buf[nepim_global.tcp_read_size];
   int rd;
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
 
   rd = read(sd, buf, nepim_global.tcp_read_size);
   if (rd < 1) {
@@ -66,7 +72,7 @@ static void *on_tcp_read(oop_source *src, int sd,
 
     fprintf(stderr, "read: connection lost on TCP socket %d\n", sd);
 
-    if (!pipe->duration_done)
+    if (!session->duration_done)
       report_broken_pipe_stat(stdout, pipe);
 
     tcp_pipe_kill(sd);
@@ -74,10 +80,10 @@ static void *on_tcp_read(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  pipe->byte_total_recv += rd;
-  pipe->byte_interval_recv += rd;
-  ++pipe->total_reads;
-  ++pipe->interval_reads;
+  session->byte_total_recv += rd;
+  session->byte_interval_recv += rd;
+  ++session->total_reads;
+  ++session->interval_reads;
 
   return OOP_CONTINUE;
 }
@@ -88,6 +94,7 @@ static void *on_tcp_write(oop_source *src, int sd,
   char buf[nepim_global.tcp_write_size];
   int wr;
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
 
   wr = write(sd, buf, nepim_global.tcp_write_size);
   if (wr < 1) {
@@ -105,7 +112,7 @@ static void *on_tcp_write(oop_source *src, int sd,
 
     fprintf(stderr, "write: connection lost on TCP socket %d\n", sd);
 
-    if (!pipe->duration_done)
+    if (!session->duration_done)
       report_broken_pipe_stat(stdout, pipe);
 
     tcp_pipe_kill(sd);
@@ -113,10 +120,10 @@ static void *on_tcp_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  pipe->byte_total_sent += wr;
-  pipe->byte_interval_sent += wr;
-  ++pipe->total_writes;
-  ++pipe->interval_writes;
+  session->byte_total_sent += wr;
+  session->byte_interval_sent += wr;
+  ++session->total_writes;
+  ++session->interval_writes;
 
   return OOP_CONTINUE;
 }
@@ -127,13 +134,14 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   char buf[nepim_global.tcp_write_size];
   int wr;
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
   int to_write;
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
-  assert(pipe->max_bit_rate > 0);
+  assert(session->max_bit_rate > 0);
 
-  to_write = NEPIM_MIN(pipe->rate_remaining, nepim_global.tcp_write_size);
+  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.tcp_write_size);
 
   wr = write(sd, buf, to_write);
   if (wr < 1) {
@@ -151,7 +159,7 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
     fprintf(stderr, "rate_write: connection lost on TCP socket %d\n", sd);
 
-    if (!pipe->duration_done)
+    if (!session->duration_done)
       report_broken_pipe_stat(stdout, pipe);
 
     tcp_pipe_kill(sd);
@@ -159,28 +167,28 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  pipe->byte_total_sent += wr;
-  pipe->byte_interval_sent += wr;
-  pipe->rate_remaining -= wr;
-  ++pipe->total_writes;
-  ++pipe->interval_writes;
+  session->byte_total_sent += wr;
+  session->byte_interval_sent += wr;
+  session->rate_remaining -= wr;
+  ++session->total_writes;
+  ++session->interval_writes;
 
 #if 0
   fprintf(stderr, 
 	  "DEBUG %s %s %d: to_write=%d written=%d missing=%d\n",
 	  __FILE__, __PRETTY_FUNCTION__,
-	  sd, to_write, wr, pipe->rate_remaining);
+	  sd, to_write, wr, session->rate_remaining);
 #endif
 
   /* finished ? */
-  if (pipe->rate_remaining < 1) {
+  if (session->rate_remaining < 1) {
     /* stop writing */
     nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				    sd, OOP_WRITE);
 
     /* schedule for next time saved by on_tcp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  pipe->tv_rate,
+				  session->tv_rate,
 				  on_tcp_rate_delay, pipe);
   }
 
@@ -190,27 +198,28 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
   long long tmp;
 
-  assert(timercmp(&tv, &pipe->tv_rate, ==));
-  assert(pipe->max_bit_rate > 0);
+  assert(timercmp(&tv, &session->tv_rate, ==));
+  assert(session->max_bit_rate > 0);
 
   /* save next scheduling time */
   {
-    int result = gettimeofday(&pipe->tv_rate, 0);
+    int result = gettimeofday(&session->tv_rate, 0);
     assert(!result);
   }
-  pipe->tv_rate.tv_usec += pipe->write_delay;
-  if (pipe->tv_rate.tv_usec >= 1000000) {
-    pipe->tv_rate.tv_usec -= 1000000;
-    ++pipe->tv_rate.tv_sec;
+  session->tv_rate.tv_usec += session->write_delay;
+  if (session->tv_rate.tv_usec >= 1000000) {
+    session->tv_rate.tv_usec -= 1000000;
+    ++session->tv_rate.tv_sec;
   }
 
   /* calculate bytes to be written from rate */
-  tmp = pipe->max_bit_rate;
-  tmp *= pipe->write_delay;
+  tmp = session->max_bit_rate;
+  tmp *= session->write_delay;
   tmp /= 8000000;
-  pipe->rate_remaining = tmp;
+  session->rate_remaining = tmp;
 
 #if 0
   fprintf(stderr, 
@@ -219,9 +228,9 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 	  "%d * %ld / 8M = %d\n",
 	  __FILE__, __PRETTY_FUNCTION__,
 	  pipe->sd, 
-	  pipe->max_bit_rate,
-	  pipe->write_delay, 
-	  pipe->rate_remaining);
+	  session->max_bit_rate,
+	  session->write_delay, 
+	  session->rate_remaining);
 #endif
 
   /* start to write */
@@ -235,21 +244,22 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
 
-  nepim_dump_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
-		  pipe->byte_total_recv,
-		  pipe->byte_total_sent,
-		  pipe->test_duration,
-		  pipe->tv_start.tv_sec,
-		  pipe->test_duration,
-		  pipe->total_reads,
-		  pipe->total_writes);
+  nepim_pipe_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
+		  session->byte_total_recv,
+		  session->byte_total_sent,
+		  session->test_duration,
+		  session->tv_start.tv_sec,
+		  session->test_duration,
+		  session->total_reads,
+		  session->total_writes);
 
-  pipe->byte_total_recv = 0;
-  pipe->byte_total_sent = 0;
-  pipe->total_reads     = 0;
-  pipe->total_writes    = 0;
-  pipe->duration_done   = 1;
+  session->byte_total_recv = 0;
+  session->byte_total_sent = 0;
+  session->total_reads     = 0;
+  session->total_writes    = 0;
+  session->duration_done   = 1;
 
   tcp_pipe_cancel_timers(pipe->sd);
 
@@ -259,113 +269,44 @@ static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
 static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
 
-  nepim_dump_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
-		  pipe->byte_interval_recv,
-		  pipe->byte_interval_sent,
-		  pipe->stat_interval,
-		  pipe->tv_start.tv_sec,
-		  pipe->test_duration,
-		  pipe->interval_reads,
-		  pipe->interval_writes);
+  nepim_pipe_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
+		  session->byte_interval_recv,
+		  session->byte_interval_sent,
+		  session->stat_interval,
+		  session->tv_start.tv_sec,
+		  session->test_duration,
+		  session->interval_reads,
+		  session->interval_writes);
 
-  pipe->byte_interval_recv = 0;
-  pipe->byte_interval_sent = 0;
-  pipe->interval_reads     = 0;
-  pipe->interval_writes    = 0;
+  session->byte_interval_recv = 0;
+  session->byte_interval_sent = 0;
+  session->interval_reads     = 0;
+  session->interval_writes    = 0;
 
   schedule_stat_interval(pipe->sd);
 
   return OOP_CONTINUE;
 }
 
-const char *SERVER_SEND   = "server_send=";
-const char *BIT_RATE      = "bit_rate=";
-const char *STAT_INTERVAL = "stat_interval=";
-const char *TEST_DURATION = "test_duration=";
-const char *WRITE_DELAY   = "write_delay=";
-
 static int parse_greetings(int sd, const char *buf, const char *past_end)
 {
-  char tmp[past_end - buf];
-  const char *SEP = " ";
-  const char *tok;
-  char *ptr;
-  int server_send    = -2;
-  long long bit_rate = -2;
-  int stat_interval  = -2;
-  int test_duration  = -2;
-  long write_delay   = -2;
-  const int SERVER_SEND_LEN   = strlen(SERVER_SEND);
-  const int BIT_RATE_LEN      = strlen(BIT_RATE);
-  const int STAT_INTERVAL_LEN = strlen(STAT_INTERVAL);
-  const int TEST_DURATION_LEN = strlen(TEST_DURATION);
-  const int WRITE_DELAY_LEN   = strlen(WRITE_DELAY);
+  nepim_greet_t opt;
+  int result;
 
-  memcpy(tmp, buf, past_end - buf);
-
-  tok = strtok_r(tmp, SEP, &ptr);
-
-  if (!tok) {
-    fprintf(stderr, "%d: bad greeting: missing first token\n",
-	    sd);
-    return -1;
-  }
-
-  if (strncmp(tok, "hello", 5)) {
-    fprintf(stderr, "%d: bad greeting: missing hello prefix\n",
-	    sd);
-    return -1;
-  }
-
-  for (;;) {
-    tok = strtok_r(0, SEP, &ptr);
-    if (!tok)
-      break;
-
-    if (!strncmp(tok, SERVER_SEND, SERVER_SEND_LEN)) {
-      server_send = atoi(tok + SERVER_SEND_LEN);
-      continue;
-    }
-
-    if (!strncmp(tok, BIT_RATE, BIT_RATE_LEN)) {
-      bit_rate = atoll(tok + BIT_RATE_LEN);
-      continue;
-    }
-
-    if (!strncmp(tok, STAT_INTERVAL, STAT_INTERVAL_LEN)) {
-      stat_interval = atoi(tok + STAT_INTERVAL_LEN);
-      continue;
-    }
-
-    if (!strncmp(tok, TEST_DURATION, TEST_DURATION_LEN)) {
-      test_duration = atoi(tok + TEST_DURATION_LEN);
-      continue;
-    }
-
-    if (!strncmp(tok, WRITE_DELAY, WRITE_DELAY_LEN)) {
-      write_delay = atoi(tok + WRITE_DELAY_LEN);
-      continue;
-    }
-
-    fprintf(stderr, "%d: bad parameter token\n", sd);
-  }
-
-  if (write_delay < 0)
-    write_delay = nepim_global.write_delay;
+  result = nepim_parse_greetings(&opt, buf, past_end);
+  if (result)
+    return result;
 
   fprintf(stderr, 
 	  "%d: TCP incoming: send=%d max_rate=%lld "
 	  "interval=%d duration=%d delay=%ld\n", 
-	  sd, server_send, bit_rate, stat_interval,
-	  test_duration, write_delay);
+	  sd, opt.must_send, opt.bit_rate, 
+	  opt.stat_interval, opt.test_duration, 
+	  opt.write_delay);
 
-  nepim_pipe_set_add(&pipes, sd, 
-		     server_send, 
-		     bit_rate, 
-		     stat_interval,
-		     test_duration,
-		     write_delay);
+  nepim_pipe_set_add(&pipes, sd, &opt);
 
   return 0;
 }
@@ -375,6 +316,7 @@ static int read_greetings(int sd)
   char buf[1024];
   int len = 0;
   char *eos;
+  int result;
 
   for (;;) {
     char *curr = buf + len;
@@ -391,7 +333,6 @@ static int read_greetings(int sd)
       return -1;
     }
     if (rd < 0) {
-
       if (errno == EAGAIN)
 	continue;
     }
@@ -408,11 +349,12 @@ static int read_greetings(int sd)
 
   *eos = '\0';
 
-  if (parse_greetings(sd, buf, eos)) {
+  result = parse_greetings(sd, buf, eos);
+  if (result) {
     fprintf(stderr, 
-	    "%s: bad client greetings: %s\n", 
-	    __PRETTY_FUNCTION__, buf);
-    return -1;
+	    "%d: bad client greetings: %d [%s]\n", 
+	    sd, result, buf);
+    return result;
   }
 
   return 0;
@@ -421,34 +363,40 @@ static int read_greetings(int sd)
 static void schedule_stat_interval(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_session_t *session;
   assert(pipe);
 
+  session = &pipe->session;
+
   {
-    int result = gettimeofday(&pipe->tv_interval, 0);
+    int result = gettimeofday(&session->tv_interval, 0);
     assert(!result);
   }
 
-  pipe->tv_interval.tv_sec += pipe->stat_interval;
+  session->tv_interval.tv_sec += session->stat_interval;
 
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				pipe->tv_interval,
+				session->tv_interval,
 				on_tcp_interval, pipe);
 }
 
 static void tcp_pipe_start(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_session_t *session;
   assert(pipe);
 
+  session = &pipe->session;
+
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
 			      sd, OOP_READ,
 			      on_tcp_read, pipe);
 
-  if (pipe->must_send) {
-    if (pipe->max_bit_rate > 0) {
-      pipe->tv_rate = OOP_TIME_NOW;
+  if (session->must_send) {
+    if (session->max_bit_rate > 0) {
+      session->tv_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    pipe->tv_rate,
+				    session->tv_rate,
 				    on_tcp_rate_delay, pipe);
     }
     else {
@@ -459,49 +407,57 @@ static void tcp_pipe_start(int sd)
   }
 
   {
-    int result = gettimeofday(&pipe->tv_start, 0);
+    int result = gettimeofday(&session->tv_start, 0);
     assert(!result);
-    pipe->tv_duration = pipe->tv_start;
+    session->tv_duration = session->tv_start;
   }
-  pipe->tv_duration.tv_sec += pipe->test_duration;
+  session->tv_duration.tv_sec += session->test_duration;
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				pipe->tv_duration,
+				session->tv_duration,
 				on_tcp_duration, pipe);
 
+  nepim_sock_show_opt(stderr, sd);
+
   schedule_stat_interval(sd);
 }
 
 static void tcp_pipe_cancel_timers(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_session_t *session;
   assert(pipe);
 
+  session = &pipe->session;
+
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    pipe->tv_duration,
+				    session->tv_duration,
 				    on_tcp_duration, pipe);
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    pipe->tv_interval,
+				    session->tv_interval,
 				    on_tcp_interval, pipe);
 }
 
 static void tcp_pipe_cancel_io(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_session_t *session;
   assert(pipe);
 
+  session = &pipe->session;
+
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				  sd, OOP_READ);
 
-  if (pipe->must_send) {
-    if (pipe->max_bit_rate > 0) {
+  if (session->must_send) {
+    if (session->max_bit_rate > 0) {
       /* stop current writing, if any */
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
 				      sd, OOP_WRITE);
       
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					pipe->tv_rate,
+					session->tv_rate,
 					on_tcp_rate_delay, pipe);
     }
     else {
@@ -519,7 +475,10 @@ static void tcp_pipe_kill(int sd)
   tcp_pipe_cancel_timers(sd);
   tcp_pipe_cancel_io(sd);
 
-  close(sd);
+  nepim_sock_show_opt(stderr, sd);
+
+  if (close(sd))
+    fprintf(stderr, "%d: close failed\n", sd);
 
   nepim_pipe_set_del(&pipes, sd);
 }
@@ -543,7 +502,8 @@ static void *on_tcp_connect(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  result = nepim_socket_opt(conn_sd);
+  result = nepim_socket_opt(conn_sd, nepim_global.pmtu_mode, 
+			    nepim_global.ttl);
   if (result) {
     fprintf(stderr, 
 	    "%s: could not set socket options: %d\n",
@@ -590,12 +550,13 @@ static void *on_tcp_connect(oop_source *src, int sd,
   return OOP_CONTINUE;
 }
 
-static void spawn_tcp_listener(const char *hostname)
+static int spawn_tcp_listener(const char *hostname, const char *portname)
 {
   struct addrinfo hints;
   struct addrinfo *ai_res;
   struct addrinfo *ai;
   int result;
+  int tcp_listeners = 0;
 
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_protocol = IPPROTO_TCP;
@@ -605,11 +566,11 @@ static void spawn_tcp_listener(const char *hostname)
   hints.ai_addr = 0;
   hints.ai_canonname = 0;
 
-  result = getaddrinfo(hostname, 0, &hints, &ai_res);
+  result = getaddrinfo(hostname, portname, &hints, &ai_res);
   if (result) {
-    fprintf(stderr, "getaddrinfo(%s): %s\n",
-            hostname, gai_strerror(result));
-    return;
+    fprintf(stderr, "getaddrinfo(%s,%s): %s\n",
+            hostname, portname, gai_strerror(result));
+    return 0;
   }
 
   for (ai = ai_res; ai; ai = ai->ai_next) {
@@ -617,19 +578,25 @@ static void spawn_tcp_listener(const char *hostname)
 
 #if 0
     fprintf(stderr, 
-	    "TCP listener: host=%s,%d len=%d family=%d type=%d proto=%d\n",
-	    hostname, nepim_global.port, 
+	    "TCP listener: host=%s,%s len=%d family=%d type=%d proto=%d\n",
+	    hostname, portname, 
 	    ai->ai_addrlen, ai->ai_family, 
 	    ai->ai_socktype, ai->ai_protocol);
 #endif
 
+    if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
+      continue;
+
     sd = nepim_create_listener_socket(ai->ai_addr, ai->ai_addrlen,
 				      ai->ai_family, ai->ai_socktype, 
-				      ai->ai_protocol, nepim_global.port);
+				      ai->ai_protocol,
+				      nepim_global.listen_backlog,
+				      nepim_global.pmtu_mode,
+				      nepim_global.ttl);
     if (sd < 0) {
       fprintf(stderr, 
-	      "%s: could not create TCP listener socket on %s,%d: %d\n",
-	      __PRETTY_FUNCTION__, hostname, nepim_global.port, sd);
+	      "%s: TCP listener socket failed for %s,%s: %d\n",
+	      __PRETTY_FUNCTION__, hostname, portname, sd);
       break;
     }
 
@@ -637,28 +604,77 @@ static void spawn_tcp_listener(const char *hostname)
 			  	sd, OOP_READ,
 				on_tcp_connect, 0);
 
+    ++tcp_listeners;
+
     fprintf(stderr, 
-	    "running TCP listener socket %d on %s,%d\n",
-	    sd, hostname, nepim_global.port);
+	    "%d: TCP socket listening on %s,%s\n",
+	    sd, hostname, portname);
   }
 
   freeaddrinfo(ai_res);
+
+  return tcp_listeners;
 }
 
-void nepim_udp_listener(const char *hostname);
+extern int nepim_udp_listener(const char *hostname,
+			      const char *portname, int join);
+
+static void parse_listeners(const char *list, int *udp, int *tcp, int join)
+{
+  int i;
+
+  int size = addr_list_size(list);
+
+  for (i = 0; i < size; ++i) {
+    char buf[100];
+    const char *portname;
+
+    if (addr_list_get(list, i, buf, sizeof(buf))) {
+      fprintf(stderr, 
+	      "%s: failure parsing address %d/%d from list: %s\n",
+	      nepim_global.prog_name, i, size, list);
+      continue;
+    }
+
+    if (addr_split_port(buf, sizeof(buf), &portname))
+      portname = nepim_global.portname;
+
+    *tcp += spawn_tcp_listener(buf, portname);
+    *udp += nepim_udp_listener(buf, portname, join);
+  }
+}
 
 void nepim_server_run()
 {
   void *result;
+  int tcp_listeners = 0;
+  int udp_listeners = 0;
 
-  if (!nepim_global.no_inet6)
-    spawn_tcp_listener(INET6_ANY);
-  spawn_tcp_listener(INET_ANY);
-  if (!nepim_global.no_inet6)
-    nepim_udp_listener(INET6_ANY);
-  nepim_udp_listener(INET_ANY);
+  if (nepim_global.bind_list)
+    parse_listeners(nepim_global.bind_list, &udp_listeners, &tcp_listeners, 0 /* join */);
 
-  nepim_pipe_set_init(&pipes);
+  if (nepim_global.join_list)
+    parse_listeners(nepim_global.join_list, &udp_listeners, &tcp_listeners, 1 /* join */);
+
+  if (!nepim_global.bind_list && !nepim_global.join_list) {
+    if (!nepim_global.no_inet6)
+      tcp_listeners += spawn_tcp_listener(INET6_ANY, nepim_global.portname);
+    tcp_listeners += spawn_tcp_listener(INET_ANY, nepim_global.portname);
+    if (!nepim_global.no_inet6)
+      udp_listeners += nepim_udp_listener(INET6_ANY, nepim_global.portname, 0 /* no join */);
+    udp_listeners += nepim_udp_listener(INET_ANY, nepim_global.portname, 0 /* no join */);
+  }
+
+  if (tcp_listeners)
+    nepim_pipe_set_init(&pipes);
+
+  if (udp_listeners) {
+    nepim_slot_set_init(&slots);
+    nepim_usock_set_init(&udp_tab);
+  }
+
+  if (tcp_listeners || udp_listeners)
+    fprintf(stderr, "%s: server ready\n", nepim_global.prog_name);
 
   result = oop_sys_run(nepim_global.oop_sys);
 }
