@@ -27,27 +27,41 @@ Boston, MA 02111-1307, USA.
 
 int nepim_socket_nonblock(int sd);
 int nepim_socket_block(int sd);
-int nepim_socket_opt(int sd);
+int nepim_socket_ttl(int sd, int ttl);
+int nepim_socket_mcast_ttl(int sd, int mcast_ttl);
+int nepim_socket_pmtu(int sd, int pmtu_mode);
+int nepim_socket_opt(int sd, int pmtu_mode, int ttl);
 int nepim_socket_tcp_opt(int sd);
 
+int nepim_socket_pmtu_get_mode(int sd);
+int nepim_socket_pmtu_get_mtu(int sd);
+int nepim_socket_get_ttl(int sd);
+
 int nepim_create_socket(struct sockaddr *addr,
 			int addr_len,
 			int family,
 			int type,
 			int protocol,
-			int port);
+			int pmtu_mode,
+			int ttl,
+			int mcast_join);
 int nepim_create_listener_socket(struct sockaddr *addr,
 				 int addr_len,
 				 int family,
 				 int type,
 				 int protocol,
-				 int port);
+				 int backlog,
+				 int pmtu_mode,
+				 int ttl);
 int nepim_connect_client_socket(struct sockaddr *addr,
 				int addr_len,
 				int family,
 				int type,
 				int protocol,
-				int port);
+				int pmtu_mode,
+				int ttl);
+
+void nepim_sock_show_opt(FILE *out, int sd);
 
 #endif /* NEPIM_SOCK_H */
 
